(* BIOS drives sector access, solyga@gmx.de, 2007..2023-03-25 *)
IMPLEMENTATION MODULE BDrvs;

IMPORT SYSTEM, SysLib, Lib, IO;

(*# save, call( o_a_copy => off ) *)

CONST
  debug = FALSE;

TYPE
  TDrvDesc = RECORD
    len: LONGCARD;   (* sector count, 0 means nok *)
  END;
  TDrvs = ARRAY TVldId OF TDrvDesc;

TYPE
  TCfgDesc = RECORD
    num: SHORTCARD;
  END;
  TCfgs = ARRAY TVldId OF TCfgDesc;

CONST
  cfgs = TCfgs(
    TCfgDesc( 80H ), TCfgDesc( 81H ), TCfgDesc( 82H ), TCfgDesc( 83H ),
    TCfgDesc( 84H ), TCfgDesc( 85H ), TCfgDesc( 86H ), TCfgDesc( 87H )
  );

TYPE
  TDskAdrPkt = RECORD
    adl: SHORTCARD;  (* SIZE(TDskAdrPkt) *)
    rsv: SHORTCARD;  (* reserved *)
    cnt: CARDINAL;   (* sector count *)
    buf: ADDRESS;    (* transfer buffer *)
    lba: LONGCARD;   (* sector address, lower 32 bits *)
    lrg: LONGCARD;   (* sector address, higher 32 bits *)
  END;

VAR
  drvs: TDrvs;


PROCEDURE Fill( VAR x: ARRAY OF BYTE; v: BYTE );
BEGIN
  Lib.Fill( ADR(x), SIZE(x), v );
END Fill;

INLINE PROCEDURE Lock();
END Lock;

INLINE PROCEDURE Unlock();
END Unlock;


PROCEDURE Len( drv: TVldId ): LONGCARD;
BEGIN
  RETURN drvs[ drv ].len;
END Len;


PROCEDURE Read( drv: TVldId; lba: LONGCARD; VAR buf: ARRAY OF BYTE; ofs, cnt: CARDINAL ): BOOLEAN;
VAR
  r: SYSTEM.Registers;
  ok: BOOLEAN;
  bc: CARDINAL;
  dap: TDskAdrPkt;

  PROCEDURE Err1();
  CONST
    ena = TRUE;
  BEGIN
    IF ~ ena THEN RETURN END;
    Lock();
    IO.WrStr( 'BDrvs.Read.Err1: ' );
    IO.WrCard( ORD(drv), 0 ); IO.WrStr( ' ' );
    IO.WrLngCard( lba, 0 ); IO.WrStr( '/' );
    IO.WrLngCard( drvs[drv].len, 0 ); IO.WrStr( ' ' );
    IO.WrCard( cnt, 0 ); IO.WrStr( ' ' );
    IO.WrCard( bc, 0 ); IO.WrStr( ' ' );
    IO.WrCard( ofs, 0 ); IO.WrStr( ' ' );
    IO.WrLn();
    Unlock();
  END Err1;

  PROCEDURE Err2();
  CONST
    ena = TRUE;
  BEGIN
    IF ~ ena THEN RETURN END;
    Lock();
    IO.WrStr( 'BDrvs.Read.Err2: ' );
    IO.WrCard( ORD(drv), 0 ); IO.WrStr( ' ' );
    IO.WrLngCard( lba, 0 ); IO.WrStr( '/' );
    IO.WrLngCard( drvs[drv].len, 0 ); IO.WrStr( ' ' );
    IO.WrCard( cnt, 0 ); IO.WrStr( ' ' );
    IO.WrCard( ORD(SYSTEM.CarryFlag IN r.Flags), 0 ); IO.WrStr( ' ' );
    IO.WrShtCard( r.AH, 0 ); IO.WrStr( ' ' );
    IO.WrCard( dap.cnt, 0 ); IO.WrStr( ' ' );
    IO.WrLn();
    Unlock();
  END Err2;

BEGIN
  IF cnt <= 0 THEN RETURN TRUE END;
  IF cnt > cntMax THEN RETURN FALSE END;
  bc := SIZE(buf) DIV lenSct;
  ok := ( bc >= ofs ) & ( bc - ofs >= cnt ) &
        ( drvs[drv].len >= lba ) & ( drvs[drv].len - lba >= LONGCARD(cnt) );
  IF ~ ok THEN Err1(); RETURN FALSE END;
  (* SysLinux mbr.asm, http://www.ctyme.com/intr/rb-0708.htm *)
  dap.adl := SHORTCARD( SIZE(dap) );
  dap.rsv := 0;
  dap.cnt := cnt;
  dap.buf := ADR( buf[ofs*lenSct] );
  dap.lba := lba;
  dap.lrg := 0;
  r.AH := 42H;
  r.DL := cfgs[ drv ].num;
  r.DS := SysLib.AdrSeg( ADR(dap) );
  r.SI := SysLib.AdrOfs( ADR(dap) );
  Lock();
  Lib.Intr( r, 13H );
  Unlock();
  ok := ~ ( SYSTEM.CarryFlag IN r.Flags ) & ( r.AH = 0 ) & ( dap.cnt = cnt );
  IF ~ ok THEN Err2() END;
  RETURN ok;
END Read;


PROCEDURE Write( drv: TVldId; lba: LONGCARD; buf: ARRAY OF BYTE; ofs, cnt: CARDINAL ): BOOLEAN;
VAR
  r: SYSTEM.Registers;
  ok: BOOLEAN;
  bc: CARDINAL;
  dap: TDskAdrPkt;

  PROCEDURE Err1();
  CONST
    ena = TRUE;
  BEGIN
    IF ~ ena THEN RETURN END;
    Lock();
    IO.WrStr( 'BDrvs.Write.Err1: ' );
    IO.WrCard( ORD(drv), 0 ); IO.WrStr( ' ' );
    IO.WrLngCard( lba, 0 ); IO.WrStr( '/' );
    IO.WrLngCard( drvs[drv].len, 0 ); IO.WrStr( ' ' );
    IO.WrCard( cnt, 0 ); IO.WrStr( ' ' );
    IO.WrCard( bc, 0 ); IO.WrStr( ' ' );
    IO.WrCard( ofs, 0 ); IO.WrStr( ' ' );
    IO.WrLn();
    Unlock();
  END Err1;

  PROCEDURE Err2();
  CONST
    ena = TRUE;
  BEGIN
    IF ~ ena THEN RETURN END;
    Lock();
    IO.WrStr( 'BDrvs.Write.Err2: ' );
    IO.WrCard( ORD(drv), 0 ); IO.WrStr( ' ' );
    IO.WrLngCard( lba, 0 ); IO.WrStr( '/' );
    IO.WrLngCard( drvs[drv].len, 0 ); IO.WrStr( ' ' );
    IO.WrCard( cnt, 0 ); IO.WrStr( ' ' );
    IO.WrCard( ORD(SYSTEM.CarryFlag IN r.Flags), 0 ); IO.WrStr( ' ' );
    IO.WrShtCard( r.AH, 0 ); IO.WrStr( ' ' );
    IO.WrCard( dap.cnt, 0 ); IO.WrStr( ' ' );
    IO.WrLn();
    Unlock();
  END Err2;

BEGIN
  IF cnt <= 0 THEN RETURN TRUE END;
  IF cnt > cntMax THEN RETURN FALSE END;
  bc := SIZE(buf) DIV lenSct;
  ok := ( bc >= ofs ) & ( bc - ofs >= cnt ) &
        ( drvs[drv].len >= lba ) & ( drvs[drv].len - lba >= LONGCARD(cnt) );
  IF ~ ok THEN Err1(); RETURN FALSE END;
  (* http://www.ctyme.com/intr/rb-0710.htm *)
  dap.adl := SHORTCARD( SIZE(dap) );
  dap.rsv := 0;
  dap.cnt := cnt;
  dap.buf := ADR( buf[ofs*lenSct] );
  dap.lba := lba;
  dap.lrg := 0;
  r.AH := 43H;
  r.DL := cfgs[ drv ].num;
  r.DS := SysLib.AdrSeg( ADR(dap) );
  r.SI := SysLib.AdrOfs( ADR(dap) );
  Lock();
  Lib.Intr( r, 13H );
  Unlock();
  ok := ~ ( SYSTEM.CarryFlag IN r.Flags ) & ( r.AH = 0 ) & ( dap.cnt = cnt );
  IF ~ ok THEN Err2() END;
  RETURN ok;
END Write;


(***** init *****)


PROCEDURE InitDrive( drv: TVldId );
TYPE
  TParDesc = RECORD
    dpl: CARDINAL;  (* SIZE(TParDesc) = 26 for v1.x *)
    inf: BITSET;
    cyl: LONGCARD;
    hds: LONGCARD;
    spt: LONGCARD;
    len: LONGCARD;  (* sectors, lower 32 bits *)
    lrg: LONGCARD;  (* sectors, higher 32 bits *)
    bps: CARDINAL;  (* bytes per sector *)
  END;
VAR
  r : SYSTEM.Registers;
  p : TParDesc;
  ok: BOOLEAN;

  PROCEDURE Err1();
  CONST
    ena = debug;
  BEGIN
    IF ~ ena THEN RETURN END;
    Lock();
    IO.WrStr( 'BDrvs.InitDrive.Err1: ' );
    IO.WrShtHex( cfgs[drv].num, 2 ); IO.WrStr( 'H ' );
    IO.WrCard( ORD(SYSTEM.CarryFlag IN r.Flags), 0 ); IO.WrStr( ' ' );
    IO.WrHex( r.BX, 0 ); IO.WrStr( 'H ' );
    IO.WrHex( r.CX, 0 ); IO.WrStr( 'H ' );
    IO.WrLn();
    Unlock();
  END Err1;

  PROCEDURE Err2();
  CONST
    ena = debug;
  BEGIN
    IF ~ ena THEN RETURN END;
    Lock();
    IO.WrStr( 'BDrvs.InitDrive.Err2: ' );
    IO.WrShtHex( cfgs[drv].num, 2 ); IO.WrStr( 'H ' );
    IO.WrCard( ORD(SYSTEM.CarryFlag IN r.Flags), 0 ); IO.WrStr( ' ' );
    IO.WrShtCard( r.AH, 0 ); IO.WrStr( ' ' );
    IO.WrCard( p.bps, 0 );
    IO.WrLn();
    Unlock();
  END Err2;

  PROCEDURE Msg();
  CONST
    ena = debug;
  BEGIN
    IF ~ ena THEN RETURN END;
    Lock();
    IO.WrStr( 'BDrvs: id ' );
    IO.WrCard( ORD(drv), 1 );
    IO.WrStr( ' num ' );
    IO.WrShtHex( cfgs[drv].num, 2 );
    IO.WrStr( 'H len ' );
    IO.WrLngCard( p.len, 0 );
    IO.WrLn();
    Unlock();
  END Msg;

BEGIN
  Fill( drvs[drv], 0 );
  (* EBIOS install check, see SysLinux mbr.asm, http://www.ctyme.com/intr/rb-0706.htm *)
  r.AX := 4100H;
  r.BX := 55AAH;
  r.CX := 0;
  r.DL := cfgs[ drv ].num;
  r.DH := 0;
  INCL( r.Flags, SYSTEM.CarryFlag );
  Lock();
  Lib.Intr( r, 13H );
  Unlock();
  ok := ~ ( SYSTEM.CarryFlag IN r.Flags ) & ( r.BX = 0AA55H ) & ( 0 IN BITSET(r.CX) );
  IF ~ ok THEN Err1(); RETURN END;
  (* drive parameters, see http://www.ctyme.com/intr/rb-0715.htm *)
  Fill( p, 0 );
  p.dpl := SIZE( p );
  r.AH := 48H;
  r.DL := cfgs[ drv ].num;
  r.DS := SysLib.AdrSeg( ADR(p) );
  r.SI := SysLib.AdrOfs( ADR(p) );
  Lock();
  Lib.Intr( r, 13H );
  Unlock();
  ok := ~ ( SYSTEM.CarryFlag IN r.Flags ) & ( r.AH = 0 ) & ( p.bps = lenSct );
  IF ~ ok THEN Err2(); RETURN END;
  drvs[ drv ].len := p.len;
  Msg();
END InitDrive;


PROCEDURE InitModule();
VAR
  drv: TVldId;
BEGIN
  Fill( drvs, 0 );
  FOR drv := MIN(TVldId) TO MAX(TVldId) DO
    InitDrive( drv );
  END;
END InitModule;

BEGIN
  InitModule();
END BDrvs.
