(* runtime config, solyga@gmx.de, 2007..2023-03-24 *)
IMPLEMENTATION MODULE Config;

IMPORT Lib, Str, IO;
IMPORT Args;


PROCEDURE ShowVerStr();
BEGIN
  IO.WrStr( prg ); IO.WrStr( ' v' ); IO.WrCard( ver, 1 ); IO.WrStr( ' (' ); IO.WrStr( ext ); IO.WrStr( '): Raw Disk Access.' );
END ShowVerStr;

PROCEDURE ShowVer();
BEGIN
  ShowVerStr();
  IO.WrLn();
END ShowVer;

PROCEDURE ParseArgs();

  PROCEDURE Help();
  BEGIN
    IO.WrStr( '(* ' );
    ShowVerStr();
    IO.WrStr( ' *)' );
    IO.WrLn();
    IO.WrStr( 'author = "solyga@gmx.de" .' );
    IO.WrLn();
    IO.WrStr( 'syntax = "' ); IO.WrStr( prg ); IO.WrStr( '" { option } drive action .' );
    IO.WrLn();
    IO.WrStr( 'option = "-h"         (* show this help and terminate *) .' );
    IO.WrLn();
    IO.WrStr( 'drive  = num          (* set drive number (0..255) *) .' );
    IO.WrLn();
    IO.WrStr( 'action = "show"       (* show drive num *) ' );
    IO.WrLn();
    IO.WrStr( '       | "read"       (* read MBR of drive num and show first bytes *) ' );
    IO.WrLn();
    IO.WrStr( '       | "write" val  (* write val to MBR of drive num *) .' );
    IO.WrLn();
  END Help;

  PROCEDURE Error( s: ARRAY OF CHAR );
  BEGIN
    IO.WrStr( 'ERROR: ' );
    IO.WrStr( s );
    IO.WrStr( ': "' );
    IO.WrStr( Args.lit );
    IO.WrChar( '"' );
    IO.WrLn();
    IO.WrStr( 'Try "' );
    IO.WrStr( prg );
    IO.WrStr( ' -?" for help.' );
    IO.WrLn();
    HALT;
  END Error;

  PROCEDURE Options();
  BEGIN
    WHILE Args.sym = Args.symOpt DO
      IF Str.Compare( Args.lit, '?' ) = 0 THEN
        Help();
        HALT;
      ELSIF Str.Compare( Args.lit, 'h' ) = 0 THEN
        Help();
        HALT;
      ELSE
        Error( 'Unknown option' );
      END;
      Args.Read();
    END;
  END Options;

  PROCEDURE Number();
  VAR
    x: LONGCARD;
    ok: BOOLEAN;
  BEGIN
    IF Args.sym # Args.symArg THEN Error( 'Expected drive number' ) END;
    x := Str.StrToCard( Args.lit, 10, ok );
    IF ~ ok OR ( x > 255 ) THEN Error( 'Invalid drive number' ) END;
    data.drv := SHORTCARD( x );
    Args.Read();
  END Number;

  PROCEDURE Value();
  VAR
    x: LONGCARD;
    ok: BOOLEAN;
  BEGIN
    IF Args.sym # Args.symArg THEN Error( 'Expected value' ) END;
    x := Str.StrToCard( Args.lit, 10, ok );
    IF ~ ok OR ( x > 255 ) THEN Error( 'Invalid value' ) END;
    data.val := SHORTCARD( x );
    Args.Read();
  END Value;

  PROCEDURE Action();
  BEGIN
    IF Args.sym # Args.symArg THEN Error( 'Invalid action' ) END;
    IF Str.Compare( Args.lit, 'show' ) = 0 THEN
      data.shw := TRUE;
      Args.Read();
    ELSIF Str.Compare( Args.lit, 'read' ) = 0 THEN
      data.rd := TRUE;
      Args.Read();
    ELSIF Str.Compare( Args.lit, 'write' ) = 0 THEN
      data.wrt := TRUE;
      Args.Read();
      Options();
      Value();
    ELSE
      Error( 'Invalid action' );
    END;
  END Action;

BEGIN
  Args.Reset();
  Args.Read();
  Options();
  Number();
  Options();
  Action();
  Options();
  IF Args.sym # Args.symEOA THEN Error( 'Unexpected argument' ) END;
END ParseArgs;

BEGIN
  Lib.Fill( ADR(data), SIZE(data), 0 );

  ParseArgs();
  ShowVer();
END Config.
