(* Raw Disk Access, solyga@gmx.de, 2023-03-25 *)
MODULE RDA;

IMPORT IO;
IMPORT Config, BDrvs;

VAR
  buf: ARRAY [0..BDrvs.lenSct-1] OF BYTE;
  drv: BDrvs.TId;


PROCEDURE Fill( VAR b: ARRAY OF BYTE; v: BYTE );
VAR i: CARDINAL;
BEGIN
  FOR i := 0 TO HIGH( b ) DO b[i] := v END;
END Fill;

PROCEDURE Dump( b: ARRAY OF BYTE; n: CARDINAL );
CONST
  max = 16;
VAR
  i: CARDINAL;
  c: CHAR;
BEGIN
  IF n > SIZE( b ) THEN n := SIZE( b ) END;
  IF n > max THEN n := max END;
  IF n <= 0 THEN RETURN END;
  c := IO.PrefixChar;
  IO.PrefixChar := '0';
  FOR i := 0 TO n - 1 DO
    IO.WrChar( ' ' );
    IO.WrShtHex( SHORTCARD(b[i]), 2 );
  END;
  IO.PrefixChar := c;
END Dump;


PROCEDURE Show();

  PROCEDURE ShowDrv( id: BDrvs.TVldId );
  BEGIN
    IO.WrStr( 'Drive ' );
    IO.WrCard( ORD( id ), 1 );
    IO.WrStr( ' len ' );
    IO.WrLngCard( BDrvs.Len( id ), 1 );
    IO.WrLn();
  END ShowDrv;

VAR
  id: BDrvs.TVldId;
BEGIN
  IF drv > BDrvs.idNull THEN
    ShowDrv( drv );
    RETURN;
  END;
  FOR id := MIN( BDrvs.TVldId ) TO MAX( BDrvs.TVldId ) DO
    ShowDrv( id );
  END;
END Show;

PROCEDURE Read();
BEGIN
  IF ( drv = BDrvs.idNull ) OR ( BDrvs.Len( drv ) <= 0 ) THEN
    IO.WrStr( 'Drive ' );
    IO.WrCard( ORD( drv ), 1 );
    IO.WrStr( ' does not exist' );
    IO.WrLn();
    RETURN;
  END;
  IF ~ BDrvs.Read( drv, 0, buf, 0, 1 ) THEN
    IO.WrStr( 'Read error for drive ' );
    IO.WrCard( ORD( drv ), 1 );
    IO.WrLn();
    RETURN;
  END;
  Dump( buf, 8 );
END Read;

PROCEDURE Write();
BEGIN
  IF ( drv = BDrvs.idNull ) OR ( BDrvs.Len( drv ) <= 0 ) THEN
    IO.WrStr( 'Drive ' );
    IO.WrCard( ORD( drv ), 1 );
    IO.WrStr( ' does not exist' );
    IO.WrLn();
    RETURN;
  END;
  Fill( buf, Config.data.val );
  IF ~ BDrvs.Write( drv, 0, buf, 0, 1 ) THEN
    IO.WrStr( 'Write error for drive ' );
    IO.WrCard( ORD( drv ), 1 );
    IO.WrLn();
    RETURN;
  END;
  Fill( buf, 0 );
  Read();
END Write;

BEGIN
  Fill( buf, 0 );
  IF Config.data.drv > ORD( MAX(BDrvs.TId) ) THEN
    IO.WrStr( 'Invalid drive number ' );
    IO.WrShtCard( Config.data.drv, 1 );
    IO.WrLn();
    RETURN;
  END;
  drv := BDrvs.TId( Config.data.drv );

  IF Config.data.shw THEN
    Show();
  ELSIF Config.data.rd THEN
    Read();
  ELSIF Config.data.wrt THEN
    Write();
  END;
END RDA.
